#ifndef __LIVECODE_VIEW__
#define __LIVECODE_VIEW__

@interface LiveCodeView : UIView

// The startup message initializes the LiveCode engine and opens the embedded
// mainstack into the view ready for use.
- (void)startup;
// The shutdown message closes the embedded mainstack and finalizes the LiveCode
// engine.
- (void)shutdown;

// Use setViewBounds to change the size and position of the view. This must be
// used instead of 'setBounds' or 'setFrame'.
- (void)setViewBounds:(CGRect)bounds;
// Use viewBounds to return the size and position of the view.
- (CGRect)viewBounds;

// Use setDelegate to attach a delegate object to the view. The messages received
// by the delegate are determined by those defined by the mapMessage message.
- (void)setDelegate:(id)delegate;
// Use delegate to fetch the current delegate attached to the view.
- (id)delegate;

// Use mapMessage to register interest in a message that passes completely
// through the engine's message path.
// Here 'message' should be the name of the LiveCode message you are interested
// in (handler name).
// The 'signature' describes how to map LiveCode types to Objective-C types and
// is described below.
// The 'selector' should be the Obj-C message to send to the delegate upon receipt
// of the LiveCode message.
// In order for the mapping to be successful, a delegate object must have been
// set, it must respond to selector and the selector's argument types must be
// compatible with signature.
//
// The signature string consists of the return type followed (in order) by the
// argument types from left to right. Each type is encoded as follows:
//   v - void
//   c - char (signed 8-bit integer)
//   s - short (signed 16-bit integer)
//   i - int (signed 32-bit integer)
//   l - long (signed 32-bit integer)
//   q - long long (signed 64-bit integer)
//   C - unsigned char (unsigned 8-bit integer)
//   S - unsigned short (unsigned 16-bit integer)
//   I - unsigned int (unsigned 32-bit integer)
//   L - unsigned long (unsigned 32-bit integer)
//   Q - unsigned long long (unsigned 64-bit integer)
//   f - float (32-bit floating-point real number)
//   d - double (64-bit floating-point real number)
//   B - bool (C++ / C99 boolean value)
//   * - const char * (NUL-terminated C-string in MacRoman encoding)
//   @ - id (any Objective-C object)
//   #N - NSNumber * (an Objective-C number object)
//   #S - NSString * (an Objective-C string object)
//   #D - NSData * (an Objective-C data object)
//
// For example, consider the engine mouseUp message which takes at most one numeric
// parameter indicating the button. This can be mapped to the Objective-C method
// 'mouseUpWithButton:(NSNumber *)button' as follows:
//   mapMessage: "mouseUp" withSignature: "v#N" toSelector: @selector(mouseUpWithButton:)   
//
- (BOOL)mapMessage:(const char *)message withSignature:(const char *)signature toSelector:(SEL)selector;

// The 'post' methods post a message to the current card of the stack currently open
// in the view. The message is added to the engine's eventqueue as is processed
// the next time the engine processes events.
// Any arguments to the message must be Objective-C objects:
//   NSNumber * map to LiveCode numbers
//   NSString * map to LiveCode strings (after being converted to MacRoman encoding)
//   NSData * map to LiveCode (binary) strings (no conversion is performed)
// If a completion block is passed then it will be invoked after the message has
// been processed (and executed) by the engine. The parameter to the block is the
// value of the result variable after dispatch.
//
- (void)post:(NSString *)message;
- (void)post:(NSString *)message completion:(void (^)(id))completion;
- (void)post:(NSString *)message arguments:(id)firstArg, ...;
- (void)post:(NSString *)message completion:(void (^)(id))completion arguments:(id)firstArg, ...;
- (void)postV:(NSString *)message completion:(void (^)(id))completion arguments:(va_list)args;

@end

#ifdef __cplusplus
extern "C"
{
#endif
    
LiveCodeView *LiveCodeGetView(void);

#ifdef __cplusplus
}
#endif

#endif
